syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface JointAnimation 
/// \brief Message for a model joint animation, does not appear to be used.
/// \todo Document me.


import "time.proto";

message JointAnimation
{
  message Joint 
  {
    repeated string name  = 1;
    repeated double angle = 2;
  }

  required string model_name   = 1;
  repeated Joint joint         = 2;
  repeated Time  time          = 3;
}


