// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: gui_camera.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_gui_5fcamera_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_gui_5fcamera_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3019000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3019001 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "pose.pb.h"
#include "track_visual.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_gui_5fcamera_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_gui_5fcamera_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxiliaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const uint32_t offsets[];
};
GZ_MSGS_VISIBLE extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_gui_5fcamera_2eproto;
namespace gazebo {
namespace msgs {
class GUICamera;
struct GUICameraDefaultTypeInternal;
GZ_MSGS_VISIBLE extern GUICameraDefaultTypeInternal _GUICamera_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::GUICamera* Arena::CreateMaybeMessage<::gazebo::msgs::GUICamera>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

// ===================================================================

class GZ_MSGS_VISIBLE GUICamera final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.GUICamera) */ {
 public:
  inline GUICamera() : GUICamera(nullptr) {}
  ~GUICamera() override;
  explicit constexpr GUICamera(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  GUICamera(const GUICamera& from);
  GUICamera(GUICamera&& from) noexcept
    : GUICamera() {
    *this = ::std::move(from);
  }

  inline GUICamera& operator=(const GUICamera& from) {
    CopyFrom(from);
    return *this;
  }
  inline GUICamera& operator=(GUICamera&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const GUICamera& default_instance() {
    return *internal_default_instance();
  }
  static inline const GUICamera* internal_default_instance() {
    return reinterpret_cast<const GUICamera*>(
               &_GUICamera_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(GUICamera& a, GUICamera& b) {
    a.Swap(&b);
  }
  inline void Swap(GUICamera* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(GUICamera* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  GUICamera* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<GUICamera>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const GUICamera& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const GUICamera& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(GUICamera* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.GUICamera";
  }
  protected:
  explicit GUICamera(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kNameFieldNumber = 1,
    kViewControllerFieldNumber = 2,
    kProjectionTypeFieldNumber = 5,
    kPoseFieldNumber = 3,
    kTrackFieldNumber = 4,
  };
  // required string name = 1;
  bool has_name() const;
  private:
  bool _internal_has_name() const;
  public:
  void clear_name();
  const std::string& name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_name();
  PROTOBUF_NODISCARD std::string* release_name();
  void set_allocated_name(std::string* name);
  private:
  const std::string& _internal_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_name(const std::string& value);
  std::string* _internal_mutable_name();
  public:

  // optional string view_controller = 2;
  bool has_view_controller() const;
  private:
  bool _internal_has_view_controller() const;
  public:
  void clear_view_controller();
  const std::string& view_controller() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_view_controller(ArgT0&& arg0, ArgT... args);
  std::string* mutable_view_controller();
  PROTOBUF_NODISCARD std::string* release_view_controller();
  void set_allocated_view_controller(std::string* view_controller);
  private:
  const std::string& _internal_view_controller() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_view_controller(const std::string& value);
  std::string* _internal_mutable_view_controller();
  public:

  // optional string projection_type = 5;
  bool has_projection_type() const;
  private:
  bool _internal_has_projection_type() const;
  public:
  void clear_projection_type();
  const std::string& projection_type() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_projection_type(ArgT0&& arg0, ArgT... args);
  std::string* mutable_projection_type();
  PROTOBUF_NODISCARD std::string* release_projection_type();
  void set_allocated_projection_type(std::string* projection_type);
  private:
  const std::string& _internal_projection_type() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_projection_type(const std::string& value);
  std::string* _internal_mutable_projection_type();
  public:

  // optional .gazebo.msgs.Pose pose = 3;
  bool has_pose() const;
  private:
  bool _internal_has_pose() const;
  public:
  void clear_pose();
  const ::gazebo::msgs::Pose& pose() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Pose* release_pose();
  ::gazebo::msgs::Pose* mutable_pose();
  void set_allocated_pose(::gazebo::msgs::Pose* pose);
  private:
  const ::gazebo::msgs::Pose& _internal_pose() const;
  ::gazebo::msgs::Pose* _internal_mutable_pose();
  public:
  void unsafe_arena_set_allocated_pose(
      ::gazebo::msgs::Pose* pose);
  ::gazebo::msgs::Pose* unsafe_arena_release_pose();

  // optional .gazebo.msgs.TrackVisual track = 4;
  bool has_track() const;
  private:
  bool _internal_has_track() const;
  public:
  void clear_track();
  const ::gazebo::msgs::TrackVisual& track() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::TrackVisual* release_track();
  ::gazebo::msgs::TrackVisual* mutable_track();
  void set_allocated_track(::gazebo::msgs::TrackVisual* track);
  private:
  const ::gazebo::msgs::TrackVisual& _internal_track() const;
  ::gazebo::msgs::TrackVisual* _internal_mutable_track();
  public:
  void unsafe_arena_set_allocated_track(
      ::gazebo::msgs::TrackVisual* track);
  ::gazebo::msgs::TrackVisual* unsafe_arena_release_track();

  // @@protoc_insertion_point(class_scope:gazebo.msgs.GUICamera)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr name_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr view_controller_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr projection_type_;
  ::gazebo::msgs::Pose* pose_;
  ::gazebo::msgs::TrackVisual* track_;
  friend struct ::TableStruct_gui_5fcamera_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// GUICamera

// required string name = 1;
inline bool GUICamera::_internal_has_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool GUICamera::has_name() const {
  return _internal_has_name();
}
inline void GUICamera::clear_name() {
  name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& GUICamera::name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.GUICamera.name)
  return _internal_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void GUICamera::set_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.GUICamera.name)
}
inline std::string* GUICamera::mutable_name() {
  std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.GUICamera.name)
  return _s;
}
inline const std::string& GUICamera::_internal_name() const {
  return name_.Get();
}
inline void GUICamera::_internal_set_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArenaForAllocation());
}
inline std::string* GUICamera::_internal_mutable_name() {
  _has_bits_[0] |= 0x00000001u;
  return name_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArenaForAllocation());
}
inline std::string* GUICamera::release_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.GUICamera.name)
  if (!_internal_has_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  auto* p = name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (name_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void GUICamera::set_allocated_name(std::string* name) {
  if (name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), name,
      GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (name_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.GUICamera.name)
}

// optional string view_controller = 2;
inline bool GUICamera::_internal_has_view_controller() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool GUICamera::has_view_controller() const {
  return _internal_has_view_controller();
}
inline void GUICamera::clear_view_controller() {
  view_controller_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& GUICamera::view_controller() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.GUICamera.view_controller)
  return _internal_view_controller();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void GUICamera::set_view_controller(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000002u;
 view_controller_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.GUICamera.view_controller)
}
inline std::string* GUICamera::mutable_view_controller() {
  std::string* _s = _internal_mutable_view_controller();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.GUICamera.view_controller)
  return _s;
}
inline const std::string& GUICamera::_internal_view_controller() const {
  return view_controller_.Get();
}
inline void GUICamera::_internal_set_view_controller(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  view_controller_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArenaForAllocation());
}
inline std::string* GUICamera::_internal_mutable_view_controller() {
  _has_bits_[0] |= 0x00000002u;
  return view_controller_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArenaForAllocation());
}
inline std::string* GUICamera::release_view_controller() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.GUICamera.view_controller)
  if (!_internal_has_view_controller()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  auto* p = view_controller_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (view_controller_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    view_controller_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void GUICamera::set_allocated_view_controller(std::string* view_controller) {
  if (view_controller != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  view_controller_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), view_controller,
      GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (view_controller_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    view_controller_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.GUICamera.view_controller)
}

// optional .gazebo.msgs.Pose pose = 3;
inline bool GUICamera::_internal_has_pose() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || pose_ != nullptr);
  return value;
}
inline bool GUICamera::has_pose() const {
  return _internal_has_pose();
}
inline const ::gazebo::msgs::Pose& GUICamera::_internal_pose() const {
  const ::gazebo::msgs::Pose* p = pose_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Pose&>(
      ::gazebo::msgs::_Pose_default_instance_);
}
inline const ::gazebo::msgs::Pose& GUICamera::pose() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.GUICamera.pose)
  return _internal_pose();
}
inline void GUICamera::unsafe_arena_set_allocated_pose(
    ::gazebo::msgs::Pose* pose) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  pose_ = pose;
  if (pose) {
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.GUICamera.pose)
}
inline ::gazebo::msgs::Pose* GUICamera::release_pose() {
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Pose* GUICamera::unsafe_arena_release_pose() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.GUICamera.pose)
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Pose* GUICamera::_internal_mutable_pose() {
  _has_bits_[0] |= 0x00000008u;
  if (pose_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Pose>(GetArenaForAllocation());
    pose_ = p;
  }
  return pose_;
}
inline ::gazebo::msgs::Pose* GUICamera::mutable_pose() {
  ::gazebo::msgs::Pose* _msg = _internal_mutable_pose();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.GUICamera.pose)
  return _msg;
}
inline void GUICamera::set_allocated_pose(::gazebo::msgs::Pose* pose) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  if (pose) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper<
            ::PROTOBUF_NAMESPACE_ID::MessageLite>::GetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose));
    if (message_arena != submessage_arena) {
      pose = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, pose, submessage_arena);
    }
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  pose_ = pose;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.GUICamera.pose)
}

// optional .gazebo.msgs.TrackVisual track = 4;
inline bool GUICamera::_internal_has_track() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  PROTOBUF_ASSUME(!value || track_ != nullptr);
  return value;
}
inline bool GUICamera::has_track() const {
  return _internal_has_track();
}
inline const ::gazebo::msgs::TrackVisual& GUICamera::_internal_track() const {
  const ::gazebo::msgs::TrackVisual* p = track_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::TrackVisual&>(
      ::gazebo::msgs::_TrackVisual_default_instance_);
}
inline const ::gazebo::msgs::TrackVisual& GUICamera::track() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.GUICamera.track)
  return _internal_track();
}
inline void GUICamera::unsafe_arena_set_allocated_track(
    ::gazebo::msgs::TrackVisual* track) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(track_);
  }
  track_ = track;
  if (track) {
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.GUICamera.track)
}
inline ::gazebo::msgs::TrackVisual* GUICamera::release_track() {
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::TrackVisual* temp = track_;
  track_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::TrackVisual* GUICamera::unsafe_arena_release_track() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.GUICamera.track)
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::TrackVisual* temp = track_;
  track_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::TrackVisual* GUICamera::_internal_mutable_track() {
  _has_bits_[0] |= 0x00000010u;
  if (track_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::TrackVisual>(GetArenaForAllocation());
    track_ = p;
  }
  return track_;
}
inline ::gazebo::msgs::TrackVisual* GUICamera::mutable_track() {
  ::gazebo::msgs::TrackVisual* _msg = _internal_mutable_track();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.GUICamera.track)
  return _msg;
}
inline void GUICamera::set_allocated_track(::gazebo::msgs::TrackVisual* track) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(track_);
  }
  if (track) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper<
            ::PROTOBUF_NAMESPACE_ID::MessageLite>::GetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(track));
    if (message_arena != submessage_arena) {
      track = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, track, submessage_arena);
    }
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  track_ = track;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.GUICamera.track)
}

// optional string projection_type = 5;
inline bool GUICamera::_internal_has_projection_type() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool GUICamera::has_projection_type() const {
  return _internal_has_projection_type();
}
inline void GUICamera::clear_projection_type() {
  projection_type_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000004u;
}
inline const std::string& GUICamera::projection_type() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.GUICamera.projection_type)
  return _internal_projection_type();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void GUICamera::set_projection_type(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000004u;
 projection_type_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.GUICamera.projection_type)
}
inline std::string* GUICamera::mutable_projection_type() {
  std::string* _s = _internal_mutable_projection_type();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.GUICamera.projection_type)
  return _s;
}
inline const std::string& GUICamera::_internal_projection_type() const {
  return projection_type_.Get();
}
inline void GUICamera::_internal_set_projection_type(const std::string& value) {
  _has_bits_[0] |= 0x00000004u;
  projection_type_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArenaForAllocation());
}
inline std::string* GUICamera::_internal_mutable_projection_type() {
  _has_bits_[0] |= 0x00000004u;
  return projection_type_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArenaForAllocation());
}
inline std::string* GUICamera::release_projection_type() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.GUICamera.projection_type)
  if (!_internal_has_projection_type()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000004u;
  auto* p = projection_type_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (projection_type_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    projection_type_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void GUICamera::set_allocated_projection_type(std::string* projection_type) {
  if (projection_type != nullptr) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  projection_type_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), projection_type,
      GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (projection_type_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    projection_type_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.GUICamera.projection_type)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

typedef boost::shared_ptr<gazebo::msgs::GUICamera> GUICameraPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

typedef const boost::shared_ptr<gazebo::msgs::GUICamera const> ConstGUICameraPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_gui_5fcamera_2eproto
