// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: user_cmd.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_user_5fcmd_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_user_5fcmd_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3019000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3019001 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
#include "light.pb.h"
#include "model.pb.h"
#include "world_control.pb.h"
#include "wrench.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_user_5fcmd_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_user_5fcmd_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxiliaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const uint32_t offsets[];
};
GZ_MSGS_VISIBLE extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_user_5fcmd_2eproto;
namespace gazebo {
namespace msgs {
class UserCmd;
struct UserCmdDefaultTypeInternal;
GZ_MSGS_VISIBLE extern UserCmdDefaultTypeInternal _UserCmd_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::UserCmd* Arena::CreateMaybeMessage<::gazebo::msgs::UserCmd>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

enum UserCmd_Type : int {
  UserCmd_Type_MOVING = 1,
  UserCmd_Type_WORLD_CONTROL = 2,
  UserCmd_Type_WRENCH = 3,
  UserCmd_Type_SCALING = 4
};
GZ_MSGS_VISIBLE bool UserCmd_Type_IsValid(int value);
constexpr UserCmd_Type UserCmd_Type_Type_MIN = UserCmd_Type_MOVING;
constexpr UserCmd_Type UserCmd_Type_Type_MAX = UserCmd_Type_SCALING;
constexpr int UserCmd_Type_Type_ARRAYSIZE = UserCmd_Type_Type_MAX + 1;

GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* UserCmd_Type_descriptor();
template<typename T>
inline const std::string& UserCmd_Type_Name(T enum_t_value) {
  static_assert(::std::is_same<T, UserCmd_Type>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function UserCmd_Type_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    UserCmd_Type_descriptor(), enum_t_value);
}
inline bool UserCmd_Type_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, UserCmd_Type* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<UserCmd_Type>(
    UserCmd_Type_descriptor(), name, value);
}
// ===================================================================

class GZ_MSGS_VISIBLE UserCmd final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.UserCmd) */ {
 public:
  inline UserCmd() : UserCmd(nullptr) {}
  ~UserCmd() override;
  explicit constexpr UserCmd(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  UserCmd(const UserCmd& from);
  UserCmd(UserCmd&& from) noexcept
    : UserCmd() {
    *this = ::std::move(from);
  }

  inline UserCmd& operator=(const UserCmd& from) {
    CopyFrom(from);
    return *this;
  }
  inline UserCmd& operator=(UserCmd&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UserCmd& default_instance() {
    return *internal_default_instance();
  }
  static inline const UserCmd* internal_default_instance() {
    return reinterpret_cast<const UserCmd*>(
               &_UserCmd_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(UserCmd& a, UserCmd& b) {
    a.Swap(&b);
  }
  inline void Swap(UserCmd* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UserCmd* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UserCmd* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<UserCmd>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const UserCmd& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const UserCmd& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(UserCmd* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.UserCmd";
  }
  protected:
  explicit UserCmd(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef UserCmd_Type Type;
  static constexpr Type MOVING =
    UserCmd_Type_MOVING;
  static constexpr Type WORLD_CONTROL =
    UserCmd_Type_WORLD_CONTROL;
  static constexpr Type WRENCH =
    UserCmd_Type_WRENCH;
  static constexpr Type SCALING =
    UserCmd_Type_SCALING;
  static inline bool Type_IsValid(int value) {
    return UserCmd_Type_IsValid(value);
  }
  static constexpr Type Type_MIN =
    UserCmd_Type_Type_MIN;
  static constexpr Type Type_MAX =
    UserCmd_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE =
    UserCmd_Type_Type_ARRAYSIZE;
  static inline const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor*
  Type_descriptor() {
    return UserCmd_Type_descriptor();
  }
  template<typename T>
  static inline const std::string& Type_Name(T enum_t_value) {
    static_assert(::std::is_same<T, Type>::value ||
      ::std::is_integral<T>::value,
      "Incorrect type passed to function Type_Name.");
    return UserCmd_Type_Name(enum_t_value);
  }
  static inline bool Type_Parse(::PROTOBUF_NAMESPACE_ID::ConstStringParam name,
      Type* value) {
    return UserCmd_Type_Parse(name, value);
  }

  // accessors -------------------------------------------------------

  enum : int {
    kModelFieldNumber = 4,
    kLightFieldNumber = 5,
    kDescriptionFieldNumber = 2,
    kEntityNameFieldNumber = 6,
    kWorldControlFieldNumber = 7,
    kWrenchFieldNumber = 8,
    kIdFieldNumber = 1,
    kTypeFieldNumber = 3,
  };
  // repeated .gazebo.msgs.Model model = 4;
  int model_size() const;
  private:
  int _internal_model_size() const;
  public:
  void clear_model();
  ::gazebo::msgs::Model* mutable_model(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Model >*
      mutable_model();
  private:
  const ::gazebo::msgs::Model& _internal_model(int index) const;
  ::gazebo::msgs::Model* _internal_add_model();
  public:
  const ::gazebo::msgs::Model& model(int index) const;
  ::gazebo::msgs::Model* add_model();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Model >&
      model() const;

  // repeated .gazebo.msgs.Light light = 5;
  int light_size() const;
  private:
  int _internal_light_size() const;
  public:
  void clear_light();
  ::gazebo::msgs::Light* mutable_light(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Light >*
      mutable_light();
  private:
  const ::gazebo::msgs::Light& _internal_light(int index) const;
  ::gazebo::msgs::Light* _internal_add_light();
  public:
  const ::gazebo::msgs::Light& light(int index) const;
  ::gazebo::msgs::Light* add_light();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Light >&
      light() const;

  // required string description = 2;
  bool has_description() const;
  private:
  bool _internal_has_description() const;
  public:
  void clear_description();
  const std::string& description() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_description(ArgT0&& arg0, ArgT... args);
  std::string* mutable_description();
  PROTOBUF_NODISCARD std::string* release_description();
  void set_allocated_description(std::string* description);
  private:
  const std::string& _internal_description() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_description(const std::string& value);
  std::string* _internal_mutable_description();
  public:

  // optional string entity_name = 6;
  bool has_entity_name() const;
  private:
  bool _internal_has_entity_name() const;
  public:
  void clear_entity_name();
  const std::string& entity_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_entity_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_entity_name();
  PROTOBUF_NODISCARD std::string* release_entity_name();
  void set_allocated_entity_name(std::string* entity_name);
  private:
  const std::string& _internal_entity_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_entity_name(const std::string& value);
  std::string* _internal_mutable_entity_name();
  public:

  // optional .gazebo.msgs.WorldControl world_control = 7;
  bool has_world_control() const;
  private:
  bool _internal_has_world_control() const;
  public:
  void clear_world_control();
  const ::gazebo::msgs::WorldControl& world_control() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::WorldControl* release_world_control();
  ::gazebo::msgs::WorldControl* mutable_world_control();
  void set_allocated_world_control(::gazebo::msgs::WorldControl* world_control);
  private:
  const ::gazebo::msgs::WorldControl& _internal_world_control() const;
  ::gazebo::msgs::WorldControl* _internal_mutable_world_control();
  public:
  void unsafe_arena_set_allocated_world_control(
      ::gazebo::msgs::WorldControl* world_control);
  ::gazebo::msgs::WorldControl* unsafe_arena_release_world_control();

  // optional .gazebo.msgs.Wrench wrench = 8;
  bool has_wrench() const;
  private:
  bool _internal_has_wrench() const;
  public:
  void clear_wrench();
  const ::gazebo::msgs::Wrench& wrench() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Wrench* release_wrench();
  ::gazebo::msgs::Wrench* mutable_wrench();
  void set_allocated_wrench(::gazebo::msgs::Wrench* wrench);
  private:
  const ::gazebo::msgs::Wrench& _internal_wrench() const;
  ::gazebo::msgs::Wrench* _internal_mutable_wrench();
  public:
  void unsafe_arena_set_allocated_wrench(
      ::gazebo::msgs::Wrench* wrench);
  ::gazebo::msgs::Wrench* unsafe_arena_release_wrench();

  // optional uint32 id = 1;
  bool has_id() const;
  private:
  bool _internal_has_id() const;
  public:
  void clear_id();
  uint32_t id() const;
  void set_id(uint32_t value);
  private:
  uint32_t _internal_id() const;
  void _internal_set_id(uint32_t value);
  public:

  // required .gazebo.msgs.UserCmd.Type type = 3;
  bool has_type() const;
  private:
  bool _internal_has_type() const;
  public:
  void clear_type();
  ::gazebo::msgs::UserCmd_Type type() const;
  void set_type(::gazebo::msgs::UserCmd_Type value);
  private:
  ::gazebo::msgs::UserCmd_Type _internal_type() const;
  void _internal_set_type(::gazebo::msgs::UserCmd_Type value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.UserCmd)
 private:
  class _Internal;

  // helper for ByteSizeLong()
  size_t RequiredFieldsByteSizeFallback() const;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Model > model_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Light > light_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr description_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr entity_name_;
  ::gazebo::msgs::WorldControl* world_control_;
  ::gazebo::msgs::Wrench* wrench_;
  uint32_t id_;
  int type_;
  friend struct ::TableStruct_user_5fcmd_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// UserCmd

// optional uint32 id = 1;
inline bool UserCmd::_internal_has_id() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool UserCmd::has_id() const {
  return _internal_has_id();
}
inline void UserCmd::clear_id() {
  id_ = 0u;
  _has_bits_[0] &= ~0x00000010u;
}
inline uint32_t UserCmd::_internal_id() const {
  return id_;
}
inline uint32_t UserCmd::id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.id)
  return _internal_id();
}
inline void UserCmd::_internal_set_id(uint32_t value) {
  _has_bits_[0] |= 0x00000010u;
  id_ = value;
}
inline void UserCmd::set_id(uint32_t value) {
  _internal_set_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.UserCmd.id)
}

// required string description = 2;
inline bool UserCmd::_internal_has_description() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool UserCmd::has_description() const {
  return _internal_has_description();
}
inline void UserCmd::clear_description() {
  description_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& UserCmd::description() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.description)
  return _internal_description();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void UserCmd::set_description(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 description_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.UserCmd.description)
}
inline std::string* UserCmd::mutable_description() {
  std::string* _s = _internal_mutable_description();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.description)
  return _s;
}
inline const std::string& UserCmd::_internal_description() const {
  return description_.Get();
}
inline void UserCmd::_internal_set_description(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  description_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArenaForAllocation());
}
inline std::string* UserCmd::_internal_mutable_description() {
  _has_bits_[0] |= 0x00000001u;
  return description_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArenaForAllocation());
}
inline std::string* UserCmd::release_description() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.UserCmd.description)
  if (!_internal_has_description()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  auto* p = description_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (description_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    description_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void UserCmd::set_allocated_description(std::string* description) {
  if (description != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  description_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), description,
      GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (description_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    description_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.UserCmd.description)
}

// required .gazebo.msgs.UserCmd.Type type = 3;
inline bool UserCmd::_internal_has_type() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool UserCmd::has_type() const {
  return _internal_has_type();
}
inline void UserCmd::clear_type() {
  type_ = 1;
  _has_bits_[0] &= ~0x00000020u;
}
inline ::gazebo::msgs::UserCmd_Type UserCmd::_internal_type() const {
  return static_cast< ::gazebo::msgs::UserCmd_Type >(type_);
}
inline ::gazebo::msgs::UserCmd_Type UserCmd::type() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.type)
  return _internal_type();
}
inline void UserCmd::_internal_set_type(::gazebo::msgs::UserCmd_Type value) {
  assert(::gazebo::msgs::UserCmd_Type_IsValid(value));
  _has_bits_[0] |= 0x00000020u;
  type_ = value;
}
inline void UserCmd::set_type(::gazebo::msgs::UserCmd_Type value) {
  _internal_set_type(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.UserCmd.type)
}

// repeated .gazebo.msgs.Model model = 4;
inline int UserCmd::_internal_model_size() const {
  return model_.size();
}
inline int UserCmd::model_size() const {
  return _internal_model_size();
}
inline ::gazebo::msgs::Model* UserCmd::mutable_model(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.model)
  return model_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Model >*
UserCmd::mutable_model() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.UserCmd.model)
  return &model_;
}
inline const ::gazebo::msgs::Model& UserCmd::_internal_model(int index) const {
  return model_.Get(index);
}
inline const ::gazebo::msgs::Model& UserCmd::model(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.model)
  return _internal_model(index);
}
inline ::gazebo::msgs::Model* UserCmd::_internal_add_model() {
  return model_.Add();
}
inline ::gazebo::msgs::Model* UserCmd::add_model() {
  ::gazebo::msgs::Model* _add = _internal_add_model();
  // @@protoc_insertion_point(field_add:gazebo.msgs.UserCmd.model)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Model >&
UserCmd::model() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.UserCmd.model)
  return model_;
}

// repeated .gazebo.msgs.Light light = 5;
inline int UserCmd::_internal_light_size() const {
  return light_.size();
}
inline int UserCmd::light_size() const {
  return _internal_light_size();
}
inline ::gazebo::msgs::Light* UserCmd::mutable_light(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.light)
  return light_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Light >*
UserCmd::mutable_light() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.UserCmd.light)
  return &light_;
}
inline const ::gazebo::msgs::Light& UserCmd::_internal_light(int index) const {
  return light_.Get(index);
}
inline const ::gazebo::msgs::Light& UserCmd::light(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.light)
  return _internal_light(index);
}
inline ::gazebo::msgs::Light* UserCmd::_internal_add_light() {
  return light_.Add();
}
inline ::gazebo::msgs::Light* UserCmd::add_light() {
  ::gazebo::msgs::Light* _add = _internal_add_light();
  // @@protoc_insertion_point(field_add:gazebo.msgs.UserCmd.light)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Light >&
UserCmd::light() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.UserCmd.light)
  return light_;
}

// optional string entity_name = 6;
inline bool UserCmd::_internal_has_entity_name() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool UserCmd::has_entity_name() const {
  return _internal_has_entity_name();
}
inline void UserCmd::clear_entity_name() {
  entity_name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& UserCmd::entity_name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.entity_name)
  return _internal_entity_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void UserCmd::set_entity_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000002u;
 entity_name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.UserCmd.entity_name)
}
inline std::string* UserCmd::mutable_entity_name() {
  std::string* _s = _internal_mutable_entity_name();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.entity_name)
  return _s;
}
inline const std::string& UserCmd::_internal_entity_name() const {
  return entity_name_.Get();
}
inline void UserCmd::_internal_set_entity_name(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  entity_name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArenaForAllocation());
}
inline std::string* UserCmd::_internal_mutable_entity_name() {
  _has_bits_[0] |= 0x00000002u;
  return entity_name_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArenaForAllocation());
}
inline std::string* UserCmd::release_entity_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.UserCmd.entity_name)
  if (!_internal_has_entity_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  auto* p = entity_name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (entity_name_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    entity_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void UserCmd::set_allocated_entity_name(std::string* entity_name) {
  if (entity_name != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  entity_name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), entity_name,
      GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (entity_name_.IsDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited())) {
    entity_name_.Set(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), "", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.UserCmd.entity_name)
}

// optional .gazebo.msgs.WorldControl world_control = 7;
inline bool UserCmd::_internal_has_world_control() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || world_control_ != nullptr);
  return value;
}
inline bool UserCmd::has_world_control() const {
  return _internal_has_world_control();
}
inline const ::gazebo::msgs::WorldControl& UserCmd::_internal_world_control() const {
  const ::gazebo::msgs::WorldControl* p = world_control_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::WorldControl&>(
      ::gazebo::msgs::_WorldControl_default_instance_);
}
inline const ::gazebo::msgs::WorldControl& UserCmd::world_control() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.world_control)
  return _internal_world_control();
}
inline void UserCmd::unsafe_arena_set_allocated_world_control(
    ::gazebo::msgs::WorldControl* world_control) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(world_control_);
  }
  world_control_ = world_control;
  if (world_control) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.UserCmd.world_control)
}
inline ::gazebo::msgs::WorldControl* UserCmd::release_world_control() {
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::WorldControl* temp = world_control_;
  world_control_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::WorldControl* UserCmd::unsafe_arena_release_world_control() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.UserCmd.world_control)
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::WorldControl* temp = world_control_;
  world_control_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::WorldControl* UserCmd::_internal_mutable_world_control() {
  _has_bits_[0] |= 0x00000004u;
  if (world_control_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::WorldControl>(GetArenaForAllocation());
    world_control_ = p;
  }
  return world_control_;
}
inline ::gazebo::msgs::WorldControl* UserCmd::mutable_world_control() {
  ::gazebo::msgs::WorldControl* _msg = _internal_mutable_world_control();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.world_control)
  return _msg;
}
inline void UserCmd::set_allocated_world_control(::gazebo::msgs::WorldControl* world_control) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(world_control_);
  }
  if (world_control) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper<
            ::PROTOBUF_NAMESPACE_ID::MessageLite>::GetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(world_control));
    if (message_arena != submessage_arena) {
      world_control = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, world_control, submessage_arena);
    }
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  world_control_ = world_control;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.UserCmd.world_control)
}

// optional .gazebo.msgs.Wrench wrench = 8;
inline bool UserCmd::_internal_has_wrench() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || wrench_ != nullptr);
  return value;
}
inline bool UserCmd::has_wrench() const {
  return _internal_has_wrench();
}
inline const ::gazebo::msgs::Wrench& UserCmd::_internal_wrench() const {
  const ::gazebo::msgs::Wrench* p = wrench_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Wrench&>(
      ::gazebo::msgs::_Wrench_default_instance_);
}
inline const ::gazebo::msgs::Wrench& UserCmd::wrench() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.UserCmd.wrench)
  return _internal_wrench();
}
inline void UserCmd::unsafe_arena_set_allocated_wrench(
    ::gazebo::msgs::Wrench* wrench) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(wrench_);
  }
  wrench_ = wrench;
  if (wrench) {
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.UserCmd.wrench)
}
inline ::gazebo::msgs::Wrench* UserCmd::release_wrench() {
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Wrench* temp = wrench_;
  wrench_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Wrench* UserCmd::unsafe_arena_release_wrench() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.UserCmd.wrench)
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Wrench* temp = wrench_;
  wrench_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Wrench* UserCmd::_internal_mutable_wrench() {
  _has_bits_[0] |= 0x00000008u;
  if (wrench_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Wrench>(GetArenaForAllocation());
    wrench_ = p;
  }
  return wrench_;
}
inline ::gazebo::msgs::Wrench* UserCmd::mutable_wrench() {
  ::gazebo::msgs::Wrench* _msg = _internal_mutable_wrench();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.UserCmd.wrench)
  return _msg;
}
inline void UserCmd::set_allocated_wrench(::gazebo::msgs::Wrench* wrench) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(wrench_);
  }
  if (wrench) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper<
            ::PROTOBUF_NAMESPACE_ID::MessageLite>::GetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(wrench));
    if (message_arena != submessage_arena) {
      wrench = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, wrench, submessage_arena);
    }
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  wrench_ = wrench;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.UserCmd.wrench)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

typedef boost::shared_ptr<gazebo::msgs::UserCmd> UserCmdPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::gazebo::msgs::UserCmd_Type> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::gazebo::msgs::UserCmd_Type>() {
  return ::gazebo::msgs::UserCmd_Type_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

typedef const boost::shared_ptr<gazebo::msgs::UserCmd const> ConstUserCmdPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_user_5fcmd_2eproto
