syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface PropagationGrid
/// \brief Wireless propagation grid


import "propagation_particle.proto";

message PropagationGrid
{
  repeated PropagationParticle particle = 1;
}

