syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Fog 
/// \brief Message for fog data



import "color.proto";

message Fog
{
  enum FogType
  {
    NONE         = 1;
    LINEAR       = 2;
    EXPONENTIAL  = 3;
    EXPONENTIAL2 = 4;
  }
  optional FogType type = 1;
  optional Color color = 2;
  optional float density = 3;
  optional float start = 4;
  optional float end = 5;
}

